import logging
import sys


def configure_logging():
    """ Configure logging to stream to std.out """
    root = logging.getLogger()
    root.setLevel(logging.INFO)

    # handler = logging.StreamHandler(sys.stdout)
    # handler.setLevel(logging.INFO)
    # formatter = logging.Formatter('%(asctime)s,%(name)s,%(levelname)s,%(message)s')
    # handler.setFormatter(formatter)
    # root.addHandler(handler)
