/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 3.0.12
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


public class JAVASPTAGClientJNI {
  public final static native long new_AnnClient(String jarg1, String jarg2);
  public final static native void delete_AnnClient(long jarg1);
  public final static native void AnnClient_SetTimeoutMilliseconds(long jarg1, AnnClient jarg1_, int jarg2);
  public final static native void AnnClient_SetSearchParam(long jarg1, AnnClient jarg1_, String jarg2, String jarg3);
  public final static native void AnnClient_ClearSearchParam(long jarg1, AnnClient jarg1_);
  public final static native BasicResult[] AnnClient_Search(long jarg1, AnnClient jarg1_, byte[] jarg2, int jarg3, String jarg4, boolean jarg5);
  public final static native boolean AnnClient_IsConnected(long jarg1, AnnClient jarg1_);
}
