# This file was automatically generated by SWIG (http://www.swig.org).
# Version 3.0.12
#
# Do not make changes to this file unless you know what you are doing--modify
# the SWIG interface file instead.

from sys import version_info as _swig_python_version_info
if _swig_python_version_info >= (2, 7, 0):
    def swig_import_helper():
        import importlib
        pkg = __name__.rpartition('.')[0]
        mname = '.'.join((pkg, '_SPTAG')).lstrip('.')
        try:
            return importlib.import_module(mname)
        except ImportError:
            return importlib.import_module('_SPTAG')
    _SPTAG = swig_import_helper()
    del swig_import_helper
elif _swig_python_version_info >= (2, 6, 0):
    def swig_import_helper():
        from os.path import dirname
        import imp
        fp = None
        try:
            fp, pathname, description = imp.find_module('_SPTAG', [dirname(__file__)])
        except ImportError:
            import _SPTAG
            return _SPTAG
        try:
            _mod = imp.load_module('_SPTAG', fp, pathname, description)
        finally:
            if fp is not None:
                fp.close()
        return _mod
    _SPTAG = swig_import_helper()
    del swig_import_helper
else:
    import _SPTAG
del _swig_python_version_info

try:
    _swig_property = property
except NameError:
    pass  # Python < 2.2 doesn't have 'property'.

try:
    import builtins as __builtin__
except ImportError:
    import __builtin__

def _swig_setattr_nondynamic(self, class_type, name, value, static=1):
    if (name == "thisown"):
        return self.this.own(value)
    if (name == "this"):
        if type(value).__name__ == 'SwigPyObject':
            self.__dict__[name] = value
            return
    method = class_type.__swig_setmethods__.get(name, None)
    if method:
        return method(self, value)
    if (not static):
        if _newclass:
            object.__setattr__(self, name, value)
        else:
            self.__dict__[name] = value
    else:
        raise AttributeError("You cannot add attributes to %s" % self)


def _swig_setattr(self, class_type, name, value):
    return _swig_setattr_nondynamic(self, class_type, name, value, 0)


def _swig_getattr(self, class_type, name):
    if (name == "thisown"):
        return self.this.own()
    method = class_type.__swig_getmethods__.get(name, None)
    if method:
        return method(self)
    raise AttributeError("'%s' object has no attribute '%s'" % (class_type.__name__, name))


def _swig_repr(self):
    try:
        strthis = "proxy of " + self.this.__repr__()
    except __builtin__.Exception:
        strthis = ""
    return "<%s.%s; %s >" % (self.__class__.__module__, self.__class__.__name__, strthis,)

try:
    _object = object
    _newclass = 1
except __builtin__.Exception:
    class _object:
        pass
    _newclass = 0

SHARED_PTR_DISOWN = _SPTAG.SHARED_PTR_DISOWN
class AnnIndex(_object):
    __swig_setmethods__ = {}
    __setattr__ = lambda self, name, value: _swig_setattr(self, AnnIndex, name, value)
    __swig_getmethods__ = {}
    __getattr__ = lambda self, name: _swig_getattr(self, AnnIndex, name)
    __repr__ = _swig_repr

    def __init__(self, *args):
        this = _SPTAG.new_AnnIndex(*args)
        try:
            self.this.append(this)
        except __builtin__.Exception:
            self.this = this
    __swig_destroy__ = _SPTAG.delete_AnnIndex
    __del__ = lambda self: None

    def SetBuildParam(self, p_name, p_value):
        return _SPTAG.AnnIndex_SetBuildParam(self, p_name, p_value)

    def SetSearchParam(self, p_name, p_value):
        return _SPTAG.AnnIndex_SetSearchParam(self, p_name, p_value)

    def Build(self, p_data, p_num):
        return _SPTAG.AnnIndex_Build(self, p_data, p_num)

    def BuildWithMetaData(self, p_data, p_meta, p_num):
        return _SPTAG.AnnIndex_BuildWithMetaData(self, p_data, p_meta, p_num)

    def Search(self, p_data, p_resultNum):
        return _SPTAG.AnnIndex_Search(self, p_data, p_resultNum)

    def SearchWithMetaData(self, p_data, p_resultNum):
        return _SPTAG.AnnIndex_SearchWithMetaData(self, p_data, p_resultNum)

    def ReadyToServe(self):
        return _SPTAG.AnnIndex_ReadyToServe(self)

    def Save(self, p_saveFile):
        return _SPTAG.AnnIndex_Save(self, p_saveFile)

    def Add(self, p_data, p_num):
        return _SPTAG.AnnIndex_Add(self, p_data, p_num)

    def AddWithMetaData(self, p_data, p_meta, p_num):
        return _SPTAG.AnnIndex_AddWithMetaData(self, p_data, p_meta, p_num)

    def Delete(self, p_data, p_num):
        return _SPTAG.AnnIndex_Delete(self, p_data, p_num)
    if _newclass:
        Load = staticmethod(_SPTAG.AnnIndex_Load)
    else:
        Load = _SPTAG.AnnIndex_Load
AnnIndex_swigregister = _SPTAG.AnnIndex_swigregister
AnnIndex_swigregister(AnnIndex)

def AnnIndex_Load(p_loaderFile):
    return _SPTAG.AnnIndex_Load(p_loaderFile)
AnnIndex_Load = _SPTAG.AnnIndex_Load

# This file is compatible with both classic and new-style classes.


