/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 3.0.12
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


public class JAVASPTAGJNI {
  public final static native long new_AnnIndex__SWIG_0(int jarg1);
  public final static native long new_AnnIndex__SWIG_1(String jarg1, String jarg2, int jarg3);
  public final static native void delete_AnnIndex(long jarg1);
  public final static native void AnnIndex_SetBuildParam(long jarg1, AnnIndex jarg1_, String jarg2, String jarg3);
  public final static native void AnnIndex_SetSearchParam(long jarg1, AnnIndex jarg1_, String jarg2, String jarg3);
  public final static native boolean AnnIndex_Build(long jarg1, AnnIndex jarg1_, byte[] jarg2, int jarg3);
  public final static native boolean AnnIndex_BuildWithMetaData(long jarg1, AnnIndex jarg1_, byte[] jarg2, byte[] jarg3, int jarg4);
  public final static native BasicResult[] AnnIndex_Search(long jarg1, AnnIndex jarg1_, byte[] jarg2, int jarg3);
  public final static native BasicResult[] AnnIndex_SearchWithMetaData(long jarg1, AnnIndex jarg1_, byte[] jarg2, int jarg3);
  public final static native boolean AnnIndex_ReadyToServe(long jarg1, AnnIndex jarg1_);
  public final static native boolean AnnIndex_Save(long jarg1, AnnIndex jarg1_, String jarg2);
  public final static native boolean AnnIndex_Add(long jarg1, AnnIndex jarg1_, byte[] jarg2, int jarg3);
  public final static native boolean AnnIndex_AddWithMetaData(long jarg1, AnnIndex jarg1_, byte[] jarg2, byte[] jarg3, int jarg4);
  public final static native boolean AnnIndex_Delete(long jarg1, AnnIndex jarg1_, byte[] jarg2, int jarg3);
  public final static native long AnnIndex_Load(String jarg1);
  public final static native void BasicResult_VID_set(long jarg1, BasicResult jarg1_, int jarg2);
  public final static native int BasicResult_VID_get(long jarg1, BasicResult jarg1_);
  public final static native void BasicResult_Dist_set(long jarg1, BasicResult jarg1_, float jarg2);
  public final static native float BasicResult_Dist_get(long jarg1, BasicResult jarg1_);
  public final static native void BasicResult_Meta_set(long jarg1, BasicResult jarg1_, byte[] jarg2);
  public final static native byte[] BasicResult_Meta_get(long jarg1, BasicResult jarg1_);
  public final static native long new_BasicResult__SWIG_0();
  public final static native long new_BasicResult__SWIG_1(int jarg1, float jarg2);
  public final static native long new_BasicResult__SWIG_2(int jarg1, float jarg2, byte[] jarg3);
  public final static native void delete_BasicResult(long jarg1);
}
