/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 3.0.12
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */


public class BasicResult {
  private transient long swigCPtr;
  protected transient boolean swigCMemOwn;

  protected BasicResult(long cPtr, boolean cMemoryOwn) {
    swigCMemOwn = cMemoryOwn;
    swigCPtr = cPtr;
  }

  protected static long getCPtr(BasicResult obj) {
    return (obj == null) ? 0 : obj.swigCPtr;
  }

  protected void finalize() {
    delete();
  }

  public synchronized void delete() {
    if (swigCPtr != 0) {
      if (swigCMemOwn) {
        swigCMemOwn = false;
        JAVASPTAGJNI.delete_BasicResult(swigCPtr);
      }
      swigCPtr = 0;
    }
  }

  public void setVID(int value) {
    JAVASPTAGJNI.BasicResult_VID_set(swigCPtr, this, value);
  }

  public int getVID() {
    return JAVASPTAGJNI.BasicResult_VID_get(swigCPtr, this);
  }

  public void setDist(float value) {
    JAVASPTAGJNI.BasicResult_Dist_set(swigCPtr, this, value);
  }

  public float getDist() {
    return JAVASPTAGJNI.BasicResult_Dist_get(swigCPtr, this);
  }

  public void setMeta(byte[] value) {
    JAVASPTAGJNI.BasicResult_Meta_set(swigCPtr, this, value);
  }

  public byte[] getMeta() { return JAVASPTAGJNI.BasicResult_Meta_get(swigCPtr, this); }

  public BasicResult() {
    this(JAVASPTAGJNI.new_BasicResult__SWIG_0(), true);
  }

  public BasicResult(int p_vid, float p_dist) {
    this(JAVASPTAGJNI.new_BasicResult__SWIG_1(p_vid, p_dist), true);
  }

  public BasicResult(int p_vid, float p_dist, byte[] p_meta) {
    this(JAVASPTAGJNI.new_BasicResult__SWIG_2(p_vid, p_dist, p_meta), true);
  }

}
